import {Injectable} from '@angular/core'
import {ApiService} from '../../shared/service/api.service'
import {Observable} from 'rxjs'
import {Recipe, RecipeStep} from '../../shared/model/recipe.model'
import {map} from 'rxjs/operators'
import {MaterialQuantity} from '../../material/service/inventory.service'

@Injectable({
  providedIn: 'root'
})
export class RecipeService {
  constructor(
    private api: ApiService
  ) {
  }

  get all(): Observable<Recipe[]> {
    return this.api.get<Recipe[]>('/recipe')
  }

  get allSortedByCompany(): Observable<{ company: string, recipes: Recipe[] }[]> {
    return this.all.pipe(map(recipes => {
      const mapped = []
      recipes.forEach(r => {
        if (!mapped[r.company.id]) {
          mapped[r.company.id] = {company: r.company.name, recipes: []}
        }
        mapped[r.company.id].recipes.push(r)
      })
      return mapped.filter(e => e != null)  // Filter to remove empty elements in the array that appears for some reason
    }))
  }

  getById(id: number): Observable<Recipe> {
    return this.api.get<Recipe>(`/recipe/${id}`)
  }

  save(name: string, description: string, color: string, gloss: number, sample: number, approbationDate: string, remark: string, companyId: number): Observable<Recipe> {
    const body = {name, description, color, gloss, sample, remark, companyId}
    if (approbationDate) {
      // @ts-ignore
      body.approbationDate = approbationDate
    }
    return this.api.post<Recipe>('/recipe', body)
  }

  update(id: number, name: string, description: string, color: string, gloss: number, sample: number, approbationDate: string, remark: string, steps: RecipeStep[] = null) {
    const body = {id, name, description, color, gloss, sample, remark, steps}
    if (approbationDate) {
      // @ts-ignore
      body.approbationDate = approbationDate
    }
    return this.api.put<Recipe>('/recipe', body)
  }

  saveExplorerModifications(id: number, note: string, mixesLocationChange: Map<number, string>): Observable<void> {
    const body = {
      id,
      note,
      mixesLocation: {}
    }
    mixesLocationChange.forEach((l, i) => body.mixesLocation[i] = l)

    return this.api.put<void>('/recipe/public', body)
  }

  deductMixFromQuantities(mixId: number, originalQuantity: number, currentQuantity: number): Observable<MaterialQuantity[]> {
    return this.deductMix(mixId, currentQuantity / originalQuantity)
  }

  deductMix(mixId: number, ratio: number): Observable<MaterialQuantity[]> {
    const body = {id: mixId, ratio}
    return this.api.put<MaterialQuantity[]>('/inventory/deduct/mix', body)
  }
}
